/**
 * Classe abstraite de base
 * pour les matériaux physiques
 * <p><strong>Exemple :</strong>
 * {@code
 *    Mat5Javadoc eau = new Eau();
 *    eau.setTemperature(-5);
 *    System.out.println(eau.getEtatφ());
 *    → "glace" }
 * </p>
 * @version 2.4
 * @since 1.8
 * @author Léa Raya DÉCORNOD
 */
public abstract class Mat5Javadoc {

  /**
   * température la <em>plus basse</em>
   * possible : {@value}°C
   * @see <a href="https://fr.wikipedia.org/wiki/Z%C3%A9ro_absolu">documentation sur le Zéro Absolu</a>
   */
  public static final double ZERO_DEGRE_KELVIN = -273.15;

  /**temperature interne en degrés Celsius*/
  protected double temperature;
  
  /**
   * définis la température
   * @param temperature en degrés Celsius
   * @throws IllegalArgumentException si
   *     la température est &lt; 0°Kelvins
   */
  public abstract void setTemperature(double temperature);

  /**
   * donne l'état physique
   * @return l'état physique
   *     (solide/liquide/gazeux)
   *     sous la forme d'une {@link String}
   * @See {@link EauChaude5#getEtatφ()}
   */
  public abstract String getEtatφ();

  /**
   * @deprecated depuis 1.5
   * @see #getEtatφ() getEtatφ remplace
   *           déshormais cette méthode
   */
  @Deprecated
  public boolean isSolid() {
    return getEtatφ().equals("solide");   
  }
}
