package org.teavm.classlib.java.util;

import java.util.Objects;

import org.teavm.classlib.java.util.concurrent.atomic.TAtomicInteger;
import org.teavm.classlib.java.util.function.TFunction;

/**
 * ThreadLocal TeaVM implementation is too dummy
 * this on use a shared static Map
 * ({@link java.util.concurrent.ConcurrentHashMap} do not exists in TeaVM)
 * @see org.teavm.classlib.java.lang.TThreadLocal
 * @see TThreadLocal
 */
public abstract class TThreadLocal<T> {
	private static TAtomicInteger nextId = new TAtomicInteger(0);
	private int id = nextId.incrementAndGet();
	private static TMap<Integer, Object> instances = new THashMap<>();
	protected abstract T initialValue();
	@SuppressWarnings("unchecked")
	public T get() {
		return (T) computeIfAbsent(instances,
				Objects.hash(Thread.currentThread().getId(), id),
				k -> this.initialValue()
		);
	}

	private static <K, V> V computeIfAbsent(
			TMap<K,V> m,
			K key,
			TFunction<? super K, ? extends V> mappingFunction) {
		Objects.requireNonNull(mappingFunction);
		V v;
		if ((v = m.get(key)) == null) { // double lock check
			synchronized (TThreadLocal.class) {
				if ((v = m.get(key)) == null) {
					V newValue;
					if ((newValue = mappingFunction.apply(key)) != null) {
						m.put(key, newValue);
						return newValue;
					}
				}
			}
		}
		return v;
	}

}