package org.teavm.helper;

import java.io.IOException;

/**
 * implements {@link AbstractPipe}
 * defining notify/wait using teaVM's primitives
 * @author Léa Рая DÉCORNOD <decornod@unistra.fr>
 */
public class Pipe extends AbstractPipe {

	private static final long DEFAULT_WAIT_TIMEOUT = 1000L;
	private static final int DEFAULT_BUF_SIZE = 4*1024;

	/** construct a new Pipe of {@link Pipe#DEFAULT_BUF_SIZE}
	* @see AbstractPipe#AbstractPipe(int) */
	public Pipe() {
		this(DEFAULT_BUF_SIZE);
	}

	/** construct a new Pipe
	 * @param size of internal buffer
	 * @see AbstractPipe#AbstractPipe(int) */
	public Pipe(int size) {
		super(size);
	}

	@Override
	protected void notifyGotInput() {
		// cannot wait/notify as entry
		// may nor come from a teaVM's Thread
	}

	@Override
	protected void waitForInputOf() throws IOException {
		// wait 'til something's available every second
		while (available() < 1) {
			try {
				Thread.sleep(DEFAULT_WAIT_TIMEOUT);
			} catch (InterruptedException e) {
				Console.error("ERROR : Interruped: " + e.getMessage());
			}
		}
	}
}