package org.teavm.helper;

import static org.teavm.metaprogramming.Metaprogramming.emit;
import static org.teavm.metaprogramming.Metaprogramming.exit;

import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

import org.reflections.Reflections;
import org.reflections.scanners.SubTypesScanner;
import org.teavm.metaprogramming.CompileTime;
import org.teavm.metaprogramming.Meta;
import org.teavm.metaprogramming.Metaprogramming;
import org.teavm.metaprogramming.ReflectClass;
import org.teavm.metaprogramming.Value;
import org.teavm.metaprogramming.reflect.ReflectMethod;

/**
 * Scan classes at {@link CompileTime} using {@link Reflections}
 * to find all main(String...) methods
 * and produce a runtime js method running each of them
 */
@CompileTime
public class ScanMainMethods {

	private ScanMainMethods() { /*_*/ }

	@Meta
	public static native void runMainMethod(String className, String... args);

	public static void runMainMethod(Value<String> className, Value<String[]> args) {
		ClassLoader classLoader = Metaprogramming.getClassLoader();
		Reflections reflections = new Reflections("", classLoader, new SubTypesScanner(false));
		Set<String> allTypes = reflections.getAllTypes();
		Value<AtomicBoolean> found = emit(() -> new AtomicBoolean(false));
		for (String clazz : allTypes) {
			ReflectClass<?> findClass = Metaprogramming.findClass(clazz);
			ReflectMethod method = findClass.getDeclaredJMethod("main", String[].class);
			if (method != null) {
				System.err.println("register main method: "
							+ method.getDeclaringClass().getName()
							+ "#" + method.getName() + "(String[])");
				emit(() -> { if (clazz.equals(className.get())) {
					found.get().set(true);
					method.invoke(null, (Object[]) args.get());
				}});
			}
		}
		emit(() -> {
			if (!found.get().get())
				throw new RuntimeException(
						"Erreur : impossible de trouver ou de charger la classe principale "
								+ className.get(),
						new ClassNotFoundException(className.get())
				);
		});
		exit();
	}

}
