package org.teavm.helper;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import org.teavm.jso.dom.html.HTMLElement;
import org.teavm.jso.dom.xml.Text;

/**
 * {@link OutputStream} adapter for {@link HTMLElement}
 * appends a {@link Text} child and sync an internal buffer
 * with it's content
 * @author Léa Рая DÉCORNOD <decornod@unistra.fr>
 */
public class TextNodeOutputStream extends ByteArrayOutputStream {

	/** {@link Text} to sync with */
	private Text text;

	/**
	 * appends a {@link Text} child
	 * @param element any {@link HTMLElement}
	 *        that accepts Text children
	 */
	public TextNodeOutputStream(HTMLElement element) {
		super(4*10245);
		this.text = element.getOwnerDocument().createTextNode("");
		element.appendChild(text);
	}

	/** {@link OutputStream#write(int)} and {@link TextNodeOutputStream#update()} */
	@Override
	public synchronized void write(int b) {
		super.write(b);
		flush();
	}

	/** {@link OutputStream#write(byte[], int, int)} and {@link TextNodeOutputStream#update()} */
	@Override
	public synchronized void write(byte[] b, int off, int len) {
		super.write(b, off, len);
		flush();
	}

	/** sync internal buffer's content with {@link Text}'s nodeValue 
	 * @see Text#setNodeValue(String) */
	@Override
	public void flush() {
		try { super.flush(); } catch (IOException e1) {}
		try {
			String stdoutContent = toString("UTF-8");
			text.setNodeValue(stdoutContent);
		} catch (UnsupportedEncodingException e) {}
	}

}
