package org.teavm.helper;

import static org.junit.jupiter.api.Assertions.*;

import java.io.IOException;
import java.util.Scanner;

import org.junit.jupiter.api.Test;

/**
 * {@link AbstractPipe}'s {@link JUnit4} tests
 * — TODO: must test write/read across AbstractPipe.ccircularBuffer.length
 * — TODO: must test write's buffer overflow
 * @author Léa Рая DÉCORNOD <decornod@unistra.fr>
 */
public class AbstractPipeStreamTest {

	/** mock notify/wait by throwing exception if asked to wait too many times (5×) */
	private AbstractPipe newPipe() {
		return new AbstractPipe(25) {
			private int count = 0;			@Override
			protected void waitForInputOf() throws IOException {
				if (++count > 5)
					throw new IllegalStateException("waitForInputOf() called more than 5 times");
			}
			@Override
			protected void notifyGotInput() throws IOException {}
		};
	}

	/** simple write followed by read test */
	@Test
	public void testReadWrite() throws Exception {
		AbstractPipe pipe = newPipe();
		String payload = "Hello World";
		char[] buffer = new char[50]; 
		int count;
		
		pipe.getOut().write(payload);
		count = pipe.getIn().read(buffer, 0, buffer.length);

		assertEquals(payload, new String(buffer,0,count));
	}

	/** two consecutive writes followed by reads test */
	@Test
	public void testConsecutiveWrites() throws Exception {
		AbstractPipe pipe = newPipe();
		String payload[] = { "First Hello", "Second Hello" };
		char[] buffer = new char[50]; 
		int count;
		
		pipe.getOut().write(payload[0]);
		count = pipe.getIn().read(buffer, 0, buffer.length);
		
		assertEquals(payload[0], new String(buffer,0,count));

		pipe.getOut().write(payload[1]);
		count = pipe.getIn().read(buffer, 0, buffer.length);
		
		assertEquals(payload[1], new String(buffer,0,count));
	}

	/** two consecutive writes followed by two consecutive reads test */
	@Test
	public void testFractionnedWrites() throws Exception {
		AbstractPipe pipe = newPipe();
		String payload[] = { "First Hello", "Second Hello" };
		char[] buffer = new char[50]; 
		int count;
		
		pipe.getOut().write(payload[0]);
		pipe.getOut().write(payload[1]);
		count = pipe.getIn().read(buffer, 0, buffer.length);
		
		assertEquals(payload[0] + payload[1], new String(buffer,0,count));
	}

	/** simple use in {@link Scanner#nextDouble()} test */
	@Test
	public void testScanner() throws Exception {
		AbstractPipe pipe = newPipe();
		String payload = "123\n";

		pipe.getOut().write(payload);
		Scanner scanner = new Scanner(pipe.getIn());
		assertEquals((double) 123, scanner.nextDouble(), 0.01);
		scanner.close();
	}
}
